@@ -98,7 +98,7 @@ typedef struct Element {
  * Thread-local data.
  */
 
-typedef struct ThreadSpecificData {
+typedef struct {
     int nbInit;			/* Number of calls to the init proc. */
     Tcl_HashTable engineTable;	/* Map a name to a style engine. Keys are
 				 * strings, values are Tk_StyleEngine
@@ -178,9 +178,9 @@ static const Tcl_ObjType styleObjType = {
 
 void
 TkStylePkgInit(
-    TkMainInfo *mainPtr)	/* The application being created. */
+    TCL_UNUSED(TkMainInfo *))	/* The application being created. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (tsdPtr->nbInit != 0) {
@@ -233,9 +233,9 @@ TkStylePkgInit(
 
 void
 TkStylePkgFree(
-    TkMainInfo *mainPtr)	/* The application being deleted. */
+    TCL_UNUSED(TkMainInfo *))	/* The application being deleted. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_HashSearch search;
     Tcl_HashEntry *entryPtr;
@@ -264,7 +264,7 @@ TkStylePkgFree(
 
     entryPtr = Tcl_FirstHashEntry(&tsdPtr->engineTable, &search);
     while (entryPtr != NULL) {
-	enginePtr = Tcl_GetHashValue(entryPtr);
+	enginePtr = (StyleEngine *)Tcl_GetHashValue(entryPtr);
 	FreeStyleEngine(enginePtr);
 	ckfree(enginePtr);
 	entryPtr = Tcl_NextHashEntry(&search);
@@ -307,7 +307,7 @@ Tk_RegisterStyleEngine(
     Tk_StyleEngine parent)	/* The engine's parent. NULL means the default
 				 * system engine. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_HashEntry *entryPtr;
     int newEntry;
@@ -331,8 +331,8 @@ Tk_RegisterStyleEngine(
      * Allocate and intitialize a new engine.
      */
 
-    enginePtr = ckalloc(sizeof(StyleEngine));
-    InitStyleEngine(enginePtr, Tcl_GetHashKey(&tsdPtr->engineTable, entryPtr),
+    enginePtr = (StyleEngine *)ckalloc(sizeof(StyleEngine));
+    InitStyleEngine(enginePtr, (const char *)Tcl_GetHashKey(&tsdPtr->engineTable, entryPtr),
 	    (StyleEngine *) parent);
     Tcl_SetHashValue(entryPtr, enginePtr);
 
@@ -364,7 +364,7 @@ InitStyleEngine(
     StyleEngine *parentPtr)	/* The engine's parent. NULL means the default
 				 * system engine. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     int elementId;
 
@@ -389,7 +389,7 @@ InitStyleEngine(
      */
 
     if (tsdPtr->nbElements > 0) {
-	enginePtr->elements = ckalloc(
+	enginePtr->elements = (StyledElement *)ckalloc(
 		sizeof(StyledElement) * tsdPtr->nbElements);
 	for (elementId = 0; elementId < tsdPtr->nbElements; elementId++) {
 	    InitStyledElement(enginePtr->elements+elementId);
@@ -419,7 +419,7 @@ static void
 FreeStyleEngine(
     StyleEngine *enginePtr)	/* The style engine to free. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     int elementId;
 
@@ -454,7 +454,7 @@ Tk_GetStyleEngine(
     const char *name)		/* Name of the engine to retrieve. NULL or
 				 * empty means the default system engine. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_HashEntry *entryPtr;
 
@@ -467,7 +467,7 @@ Tk_GetStyleEngine(
 	return NULL;
     }
 
-    return Tcl_GetHashValue(entryPtr);
+    return (Tk_StyleEngine)Tcl_GetHashValue(entryPtr);
 }
 
 /*
@@ -521,7 +521,7 @@ InitElement(
 
 static void
 FreeElement(
-    Element *elementPtr)	/* The element to free. */
+    TCL_UNUSED(Element *))	/* The element to free. */
 {
     /* Nothing to do. */
 }
@@ -604,12 +604,12 @@ CreateElement(
 				 * created explicitly (being registered) or
 				 * implicitly (by a derived element). */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_HashEntry *entryPtr, *engineEntryPtr;
     Tcl_HashSearch search;
     int newEntry, elementId, genericId = -1;
-    char *dot;
+    const char *dot;
     StyleEngine *enginePtr;
 
     /*
@@ -642,10 +642,10 @@ CreateElement(
      * Reallocate element table.
      */
 
-    tsdPtr->elements = ckrealloc(tsdPtr->elements,
+    tsdPtr->elements = (Element *)ckrealloc(tsdPtr->elements,
 	    sizeof(Element) * tsdPtr->nbElements);
     InitElement(tsdPtr->elements+elementId,
-	    Tcl_GetHashKey(&tsdPtr->elementTable, entryPtr), elementId,
+	    (const char *)Tcl_GetHashKey(&tsdPtr->elementTable, entryPtr), elementId,
 	    genericId, create);
 
     /*
@@ -654,9 +654,9 @@ CreateElement(
 
     engineEntryPtr = Tcl_FirstHashEntry(&tsdPtr->engineTable, &search);
     while (engineEntryPtr != NULL) {
-	enginePtr = Tcl_GetHashValue(engineEntryPtr);
+	enginePtr = (StyleEngine *)Tcl_GetHashValue(engineEntryPtr);
 
-	enginePtr->elements = ckrealloc(enginePtr->elements,
+	enginePtr->elements = (StyledElement *)ckrealloc(enginePtr->elements,
 		sizeof(StyledElement) * tsdPtr->nbElements);
 	InitStyledElement(enginePtr->elements+elementId);
 
@@ -686,11 +686,11 @@ int
 Tk_GetElementId(
     const char *name)		/* Name of the element. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_HashEntry *entryPtr;
     int genericId = -1;
-    char *dot;
+    const char *dot;
 
     /*
      * Find the element Id.
@@ -759,7 +759,7 @@ Tk_RegisterStyledElement(
     StyledElement *elementPtr;
     Tk_ElementSpec *specPtr;
     int nbOptions;
-    register Tk_ElementOptionSpec *srcOptions, *dstOptions;
+    Tk_ElementOptionSpec *srcOptions, *dstOptions;
 
     if (templatePtr->version != TK_STYLE_VERSION_1) {
 	/*
@@ -786,16 +786,16 @@ Tk_RegisterStyledElement(
 
     elementPtr = ((StyleEngine *) engine)->elements+elementId;
 
-    specPtr = ckalloc(sizeof(Tk_ElementSpec));
+    specPtr = (Tk_ElementSpec *)ckalloc(sizeof(Tk_ElementSpec));
     specPtr->version = templatePtr->version;
-    specPtr->name = ckalloc(strlen(templatePtr->name)+1);
+    specPtr->name = (char *)ckalloc(strlen(templatePtr->name)+1);
     strcpy(specPtr->name, templatePtr->name);
     nbOptions = 0;
     for (nbOptions = 0, srcOptions = templatePtr->options;
 	    srcOptions->name != NULL; nbOptions++, srcOptions++) {
 	/* empty body */
     }
-    specPtr->options =
+    specPtr->options = (Tk_ElementOptionSpec *)
 	    ckalloc(sizeof(Tk_ElementOptionSpec) * (nbOptions+1));
     for (srcOptions = templatePtr->options, dstOptions = specPtr->options;
 	    /* End condition within loop */; srcOptions++, dstOptions++) {
@@ -804,7 +804,7 @@ Tk_RegisterStyledElement(
 	    break;
 	}
 
-	dstOptions->name = ckalloc(strlen(srcOptions->name)+1);
+	dstOptions->name = (char *)ckalloc(strlen(srcOptions->name)+1);
 	strcpy(dstOptions->name, srcOptions->name);
 	dstOptions->type = srcOptions->type;
     }
@@ -844,7 +844,7 @@ GetStyledElement(
     int elementId)		/* Unique element ID */
 {
     StyledElement *elementPtr;
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     StyleEngine *enginePtr2;
 
@@ -924,7 +924,7 @@ InitWidgetSpec(
      */
 
     widgetSpecPtr->optionsPtr =
-	    ckalloc(sizeof(Tk_OptionSpec *) * nbOptions);
+	    (const Tk_OptionSpec **)ckalloc(sizeof(Tk_OptionSpec *) * nbOptions);
     for (i = 0, elementOptionPtr = elementPtr->specPtr->options;
 	    i < nbOptions; i++, elementOptionPtr++) {
 	widgetOptionPtr = TkGetOptionSpec(elementOptionPtr->name, optionTable);
@@ -1008,7 +1008,7 @@ GetWidgetSpec(
      */
 
     i = elementPtr->nbWidgetSpecs++;
-    elementPtr->widgetSpecs = ckrealloc(elementPtr->widgetSpecs,
+    elementPtr->widgetSpecs = (StyledWidgetSpec *)ckrealloc(elementPtr->widgetSpecs,
 	    sizeof(StyledWidgetSpec) * elementPtr->nbWidgetSpecs);
     widgetSpecPtr = elementPtr->widgetSpecs+i;
     InitWidgetSpec(widgetSpecPtr, elementPtr, optionTable);
@@ -1229,7 +1229,7 @@ Tk_CreateStyle(
     Tk_StyleEngine engine,	/* The style engine. */
     ClientData clientData)	/* Private data passed as is to engine code. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_HashEntry *entryPtr;
     int newEntry;
@@ -1253,8 +1253,8 @@ Tk_CreateStyle(
      * Allocate and intitialize a new style.
      */
 
-    stylePtr = ckalloc(sizeof(Style));
-    InitStyle(stylePtr, Tcl_GetHashKey(&tsdPtr->styleTable, entryPtr),
+    stylePtr = (Style *)ckalloc(sizeof(Style));
+    InitStyle(stylePtr, (const char *)Tcl_GetHashKey(&tsdPtr->styleTable, entryPtr),
 	    (engine!=NULL ? (StyleEngine*) engine : tsdPtr->defaultEnginePtr),
 	    clientData);
     Tcl_SetHashValue(entryPtr, stylePtr);
@@ -1344,10 +1344,9 @@ Tk_GetStyle(
     const char *name)		/* Name of the style to retrieve. NULL or empty
 				 * means the default system style. */
 {
-    ThreadSpecificData *tsdPtr =
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_HashEntry *entryPtr;
-    Style *stylePtr;
 
     /*
      * Search for a corresponding entry in the style table.
@@ -1360,11 +1359,9 @@ Tk_GetStyle(
 		    "style \"%s\" doesn't exist", name));
 	    Tcl_SetErrorCode(interp, "TK", "LOOKUP", "STYLE", name, NULL);
 	}
-	return (Tk_Style) NULL;
+	return NULL;
     }
-    stylePtr = Tcl_GetHashValue(entryPtr);
-
-    return (Tk_Style) stylePtr;
+    return (Tk_Style)Tcl_GetHashValue(entryPtr);
 }
 
 /*
@@ -1379,7 +1376,7 @@ Tk_GetStyle(
 
 void
 Tk_FreeStyle(
-    Tk_Style style)
+    TCL_UNUSED(Tk_Style))
 {
 }
 
@@ -1456,7 +1453,7 @@ Tk_GetStyleFromObj(
  */
 void
 Tk_FreeStyleFromObj(
-    Tcl_Obj *objPtr)
+    TCL_UNUSED(Tcl_Obj *))
 {
 }
 
