@@ -10,8 +10,7 @@
  * [style map].
  */
 
-#include <string.h>
-#include <tk.h>
+#include "tkInt.h"
 #include "ttkTheme.h"
 
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
@@ -25,6 +24,8 @@ struct TtkImageSpec {
     int 		mapCount;	/* #state-specific overrides */
     Ttk_StateSpec	*states;	/* array[mapCount] of states ... */
     Tk_Image		*images;	/* ... per-state images to use */
+    Tk_ImageChangedProc *imageChanged;
+    ClientData		imageChangedClientData;
 };
 
 /* NullImageChanged --
@@ -34,14 +35,40 @@ static void NullImageChanged(ClientData clientData,
     int x, int y, int width, int height, int imageWidth, int imageHeight)
 { /* No-op */ }
 
+/* ImageSpecImageChanged --
+ *     Image changes should trigger a repaint.
+ */
+static void ImageSpecImageChanged(ClientData clientData,
+    int x, int y, int width, int height, int imageWidth, int imageHeight)
+{
+    Ttk_ImageSpec *imageSpec = (Ttk_ImageSpec *)clientData;
+    if (imageSpec->imageChanged != NULL) {
+	imageSpec->imageChanged(imageSpec->imageChangedClientData,
+		x, y, width, height,
+		imageWidth, imageHeight);
+    }
+}
+
 /* TtkGetImageSpec --
  * 	Constructs a Ttk_ImageSpec * from a Tcl_Obj *.
  * 	Result must be released using TtkFreeImageSpec.
  *
- * TODO: Need a variant of this that takes a user-specified ImageChanged proc
  */
 Ttk_ImageSpec *
 TtkGetImageSpec(Tcl_Interp *interp, Tk_Window tkwin, Tcl_Obj *objPtr)
+{
+    return TtkGetImageSpecEx(interp, tkwin, objPtr, NULL, NULL);
+}
+
+/* TtkGetImageSpecEx --
+ * 	Constructs a Ttk_ImageSpec * from a Tcl_Obj *.
+ * 	Result must be released using TtkFreeImageSpec.
+ * 	imageChangedProc will be called when not NULL when
+ * 	the image changes to allow widgets to repaint.
+ */
+Ttk_ImageSpec *
+TtkGetImageSpecEx(Tcl_Interp *interp, Tk_Window tkwin, Tcl_Obj *objPtr,
+    Tk_ImageChangedProc *imageChangedProc, ClientData imageChangedClientData)
 {
     Ttk_ImageSpec *imageSpec = 0;
     int i = 0, n = 0, objc;
@@ -52,6 +79,8 @@ TtkGetImageSpec(Tcl_Interp *interp, Tk_Window tkwin, Tcl_Obj *objPtr)
     imageSpec->mapCount = 0;
     imageSpec->states = 0;
     imageSpec->images = 0;
+    imageSpec->imageChanged = imageChangedProc;
+    imageSpec->imageChangedClientData = imageChangedClientData;
 
     if (Tcl_ListObjGetElements(interp, objPtr, &objc, &objv) != TCL_OK) {
 	goto error;
@@ -74,7 +103,7 @@ TtkGetImageSpec(Tcl_Interp *interp, Tk_Window tkwin, Tcl_Obj *objPtr)
     /* Get base image:
     */
     imageSpec->baseImage = Tk_GetImage(
-	    interp, tkwin, Tcl_GetString(objv[0]), NullImageChanged, NULL);
+	    interp, tkwin, Tcl_GetString(objv[0]), ImageSpecImageChanged, imageSpec);
     if (!imageSpec->baseImage) {
     	goto error;
     }
