@@ -39,12 +39,13 @@ proc ::tk::TearOffMenu {w {x 0} {y 0}} {
 	    # Shift by height of tearoff entry minus height of window titlebar
 	    catch {incr y [expr {[$w yposition 1] - 16}]}
 	    # Avoid the native menu bar which sits on top of everything.
-	    if {$y < 22} { set y 22 }
+	    if {$y < 22} {set y 22}
 	}
     }
 
     set parent [winfo parent $w]
-    while {[winfo toplevel $parent] ne $parent || [winfo class $parent] eq "Menu"} {
+    while {[winfo toplevel $parent] ne $parent \
+	    || [winfo class $parent] eq "Menu"} {
 	set parent [winfo parent $parent]
     }
     if {$parent eq "."} {
@@ -67,7 +68,7 @@ proc ::tk::TearOffMenu {w {x 0} {y 0}} {
     if {[$menu cget -title] ne ""} {
     	wm title $menu [$menu cget -title]
     } else {
-    	switch [winfo class $parent] {
+    	switch -- [winfo class $parent] {
 	    Menubutton {
 	    	wm title $menu [$parent cget -text]
 	    }
@@ -77,6 +78,16 @@ proc ::tk::TearOffMenu {w {x 0} {y 0}} {
 	}
     }
 
+    if {[tk windowingsystem] eq "win32"} {
+        # [Bug 3181181]: Find the toplevel window for the menu
+        set parent [winfo toplevel $parent]
+        while {[winfo class $parent] eq "Menu"} {
+            set parent [winfo toplevel [winfo parent $parent]]
+        }
+	wm transient $menu [winfo toplevel $parent]
+	wm attributes $menu -toolwindow 1
+    }
+
     $menu post $x $y
 
     if {[winfo exists $menu] == 0} {
@@ -139,12 +150,14 @@ proc ::tk::MenuDup {src dst type} {
 
     set tags [bindtags $src]
     set srcLen [string length $src]
- 
+
     # Copy tags to x, replacing each substring of src with dst.
 
-    while {[set index [string first $src $tags]] != -1} {
-	append x [string range $tags 0 [expr {$index - 1}]]$dst
-	set tags [string range $tags [expr {$index + $srcLen}] end]
+    while {[set index [string first $src $tags]] >= 0} {
+	if {$index > 0} {
+	    append x [string range $tags 0 $index-1]$dst
+	}
+	set tags [string range $tags $index+$srcLen end]
     }
     append x $tags
 
@@ -157,10 +170,12 @@ proc ::tk::MenuDup {src dst type} {
 
 	# Copy script to x, replacing each substring of event with dst.
 
-	while {[set index [string first $event $script]] != -1} {
-	    append x [string range $script 0 [expr {$index - 1}]]
+	while {[set index [string first $event $script]] >= 0} {
+	    if {$index > 0} {
+		append x [string range $script 0 $index-1]
+	    }
 	    append x $dst
-	    set script [string range $script [expr {$index + $eventLen}] end]
+	    set script [string range $script $index+$eventLen end]
 	}
 	append x $script
 
