@@ -23,7 +23,9 @@
 /*
  * The definitions below provide symbolic names for the default colors.
  * NORMAL_BG -		Normal background color.
+ * NORMAL_FG -		Normal foreground color.
  * ACTIVE_BG -		Background color when widget is active.
+ * ACTIVE_FG -		Foreground color when widget is active.
  * SELECT_BG -		Background color for selected text.
  * SELECT_FG -		Foreground color for selected text.
  * TROUGH -		Background color for troughs in scales and scrollbars.
@@ -33,15 +35,18 @@
 
 #define BLACK			"Black"
 #define WHITE			"White"
-#define NORMAL_BG		"systemWindowBody"
-#define ACTIVE_BG		"systemButtonFacePressed"
-#define ACTIVE_FG		"systemPushButtonPressedText"
-#define SELECT_BG		"systemHighlight"
-#define SELECT_FG		None
-#define INACTIVE_SELECT_BG	"systemHighlightSecondary"
+#define NORMAL_BG		"systemWindowBackgroundColor"
+#define TEXT_BG                 "systemTextBackgroundColor"
+#define NORMAL_FG		"systemTextColor"
+#define ACTIVE_BG		"systemWindowBackgroundColor"
+#define ACTIVE_FG		"systemTextColor"
+#define SELECT_BG		"systemSelectedTextBackgroundColor"
+#define SELECT_FG		"systemSelectedTextColor"
+#define INACTIVE_SELECT_BG	"systemUnemphasizedSelectedTextBackgroundColor"
 #define TROUGH			"#c3c3c3"
 #define INDICATOR		"#b03060"
 #define DISABLED		"#a3a3a3"
+#define IGNORED                 "#abcdef"
 
 /*
  * Defaults for labels, buttons, checkbuttons, and radiobuttons:
@@ -50,11 +55,10 @@
 #define DEF_BUTTON_ANCHOR		"center"
 #define DEF_BUTTON_ACTIVE_BG_COLOR	ACTIVE_BG
 #define DEF_BUTTON_ACTIVE_BG_MONO	BLACK
-#define DEF_BUTTON_ACTIVE_FG_COLOR	ACTIVE_FG
-#define DEF_CHKRAD_ACTIVE_FG_COLOR	DEF_BUTTON_ACTIVE_FG_COLOR
+#define DEF_BUTTON_ACTIVE_FG_COLOR	WHITE
+#define DEF_CHKRAD_ACTIVE_FG_COLOR	ACTIVE_FG
 #define DEF_BUTTON_ACTIVE_FG_MONO	WHITE
-/* #define DEF_BUTTON_BG_COLOR	"systemButtonFace"*/
-#define DEF_BUTTON_BG_COLOR		WHITE
+#define DEF_BUTTON_BG_COLOR		NORMAL_BG
 #define DEF_BUTTON_BG_MONO		WHITE
 #define DEF_BUTTON_BITMAP		""
 #define DEF_BUTTON_BORDER_WIDTH		"2"
@@ -64,7 +68,7 @@
 #define DEF_BUTTON_DEFAULT		"disabled"
 #define DEF_BUTTON_DISABLED_FG_COLOR	DISABLED
 #define DEF_BUTTON_DISABLED_FG_MONO	""
-#define DEF_BUTTON_FG			"systemButtonText"
+#define DEF_BUTTON_FG			NORMAL_FG
 #define DEF_CHKRAD_FG			DEF_BUTTON_FG
 #define DEF_BUTTON_FONT			"TkDefaultFont"
 #define DEF_BUTTON_HEIGHT		"0"
@@ -78,26 +82,16 @@
 //#else
 #define DEF_BUTTON_HIGHLIGHT_WIDTH	"1"
 //#endif
-#define DEF_BUTTON_IMAGE		((char *) NULL)
+#define DEF_BUTTON_IMAGE		NULL
 #define DEF_BUTTON_INDICATOR		"1"
 #define DEF_BUTTON_JUSTIFY		"center"
 #define DEF_BUTTON_OFF_VALUE		"0"
 #define DEF_BUTTON_ON_VALUE		"1"
 #define DEF_BUTTON_TRISTATE_VALUE	""
 #define DEF_BUTTON_OVER_RELIEF		""
-//#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
-//#define DEF_BUTTON_PADX			"12"
-//#define DEF_BUTTON_PADX_NOCM		"1"
-//#else
 #define DEF_BUTTON_PADX			"1"
-//#endif
 #define DEF_LABCHKRAD_PADX		"1"
-//#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
-//#define DEF_BUTTON_PADY			"3"
-//#define DEF_BUTTON_PADY_NOCM		"1"
-//#else
 #define DEF_BUTTON_PADY			"1"
-//#endif
 #define DEF_LABCHKRAD_PADY		"1"
 #define DEF_BUTTON_RELIEF		"flat"
 #define DEF_LABCHKRAD_RELIEF		"flat"
@@ -105,10 +99,10 @@
 #define DEF_BUTTON_REPEAT_INTERVAL	"0"
 #define DEF_BUTTON_SELECT_COLOR		INDICATOR
 #define DEF_BUTTON_SELECT_MONO		BLACK
-#define DEF_BUTTON_SELECT_IMAGE		((char *) NULL)
+#define DEF_BUTTON_SELECT_IMAGE		NULL
 #define DEF_BUTTON_STATE		"normal"
 #define DEF_LABEL_TAKE_FOCUS		"0"
-#define DEF_BUTTON_TAKE_FOCUS		((char *) NULL)
+#define DEF_BUTTON_TAKE_FOCUS		NULL
 #define DEF_BUTTON_TEXT			""
 #define DEF_BUTTON_TEXT_VARIABLE	""
 #define DEF_BUTTON_UNDERLINE		"-1"
@@ -146,7 +140,7 @@
 #define DEF_CANVAS_SELECT_BD_MONO	"0"
 #define DEF_CANVAS_SELECT_FG_COLOR	SELECT_FG
 #define DEF_CANVAS_SELECT_FG_MONO	WHITE
-#define DEF_CANVAS_TAKE_FOCUS		((char *) NULL)
+#define DEF_CANVAS_TAKE_FOCUS		NULL
 #define DEF_CANVAS_WIDTH		"10c"
 #define DEF_CANVAS_X_SCROLL_CMD		""
 #define DEF_CANVAS_X_SCROLL_INCREMENT	"0"
@@ -169,7 +163,7 @@
 #define MAC_OSX_ENTRY_RELIEF		TK_RELIEF_SUNKEN
 #define MAC_OSX_ENTRY_SELECT_RELIEF	TK_RELIEF_FLAT
 
-#define DEF_ENTRY_BG_COLOR		NORMAL_BG
+#define DEF_ENTRY_BG_COLOR		TEXT_BG
 #define DEF_ENTRY_BG_MONO		WHITE
 #define DEF_ENTRY_BORDER_WIDTH		"2"
 #define DEF_ENTRY_CURSOR		"xterm"
@@ -178,23 +172,20 @@
 #define DEF_ENTRY_DISABLED_FG		DISABLED
 #define DEF_ENTRY_EXPORT_SELECTION	"1"
 #define DEF_ENTRY_FONT			"TkTextFont"
-#define DEF_ENTRY_FG			BLACK
+#define DEF_ENTRY_FG			NORMAL_FG
 #define DEF_ENTRY_HIGHLIGHT_BG		NORMAL_BG
 #define DEF_ENTRY_HIGHLIGHT		BLACK
-/* #define DEF_ENTRY_HIGHLIGHT_WIDTH	"3" */
 #define DEF_ENTRY_HIGHLIGHT_WIDTH	"3"
-#define DEF_ENTRY_INSERT_BG		BLACK
+#define DEF_ENTRY_INSERT_BG		NORMAL_FG
 #define DEF_ENTRY_INSERT_BD_COLOR	"0"
 #define DEF_ENTRY_INSERT_BD_MONO	"0"
 #define DEF_ENTRY_INSERT_OFF_TIME	"300"
 #define DEF_ENTRY_INSERT_ON_TIME	"600"
-/* #define DEF_ENTRY_INSERT_WIDTH		"2" */
 #define DEF_ENTRY_INSERT_WIDTH		"1"
 #define DEF_ENTRY_JUSTIFY		"left"
 #define DEF_ENTRY_READONLY_BG_COLOR	NORMAL_BG
 #define DEF_ENTRY_READONLY_BG_MONO	WHITE
 #define DEF_ENTRY_RELIEF		"sunken"
-/* #define DEF_ENTRY_RELIEF		"solid" */
 #define DEF_ENTRY_SCROLL_COMMAND	""
 #define DEF_ENTRY_SELECT_COLOR		SELECT_BG
 #define DEF_ENTRY_SELECT_MONO		BLACK
@@ -202,9 +193,9 @@
 #define DEF_ENTRY_SELECT_BD_MONO	"0"
 #define DEF_ENTRY_SELECT_FG_COLOR	SELECT_FG
 #define DEF_ENTRY_SELECT_FG_MONO	WHITE
-#define DEF_ENTRY_SHOW			((char *) NULL)
+#define DEF_ENTRY_SHOW			NULL
 #define DEF_ENTRY_STATE			"normal"
-#define DEF_ENTRY_TAKE_FOCUS		((char *) NULL)
+#define DEF_ENTRY_TAKE_FOCUS		NULL
 #define DEF_ENTRY_TEXT_VARIABLE		""
 #define DEF_ENTRY_WIDTH			"20"
 
@@ -237,7 +228,7 @@
 #define DEF_LABELFRAME_BORDER_WIDTH	"2"
 #define DEF_LABELFRAME_CLASS		"Labelframe"
 #define DEF_LABELFRAME_RELIEF		"groove"
-#define DEF_LABELFRAME_FG		"systemButtonText"
+#define DEF_LABELFRAME_FG		NORMAL_FG
 #define DEF_LABELFRAME_FONT		"TkDefaultFont"
 #define DEF_LABELFRAME_TEXT		""
 #define DEF_LABELFRAME_LABELANCHOR	"nw"
@@ -247,14 +238,14 @@
  */
 
 #define DEF_LISTBOX_ACTIVE_STYLE	"dotbox"
-#define DEF_LISTBOX_BG_COLOR		WHITE
+#define DEF_LISTBOX_BG_COLOR		TEXT_BG
 #define DEF_LISTBOX_BG_MONO		WHITE
 #define DEF_LISTBOX_BORDER_WIDTH	"1"
 #define DEF_LISTBOX_CURSOR		""
 #define DEF_LISTBOX_DISABLED_FG		DISABLED
 #define DEF_LISTBOX_EXPORT_SELECTION	"1"
 #define DEF_LISTBOX_FONT		"TkTextFont"
-#define DEF_LISTBOX_FG			BLACK
+#define DEF_LISTBOX_FG			NORMAL_FG
 #define DEF_LISTBOX_HEIGHT		"10"
 #define DEF_LISTBOX_HIGHLIGHT_BG	NORMAL_BG
 #define DEF_LISTBOX_HIGHLIGHT		BLACK
@@ -271,62 +262,62 @@
 #define DEF_LISTBOX_SELECT_MODE		"browse"
 #define DEF_LISTBOX_SET_GRID		"0"
 #define DEF_LISTBOX_STATE		"normal"
-#define DEF_LISTBOX_TAKE_FOCUS		((char *) NULL)
+#define DEF_LISTBOX_TAKE_FOCUS		NULL
 #define DEF_LISTBOX_WIDTH		"20"
 
 /*
  * Defaults for individual entries of menus:
  */
 
-#define DEF_MENU_ENTRY_ACTIVE_BG	((char *) NULL)
-#define DEF_MENU_ENTRY_ACTIVE_FG	((char *) NULL)
-#define DEF_MENU_ENTRY_ACCELERATOR	((char *) NULL)
-#define DEF_MENU_ENTRY_BG		((char *) NULL)
-#define DEF_MENU_ENTRY_BITMAP		None
+#define DEF_MENU_ENTRY_ACTIVE_BG	NULL
+#define DEF_MENU_ENTRY_ACTIVE_FG	NULL
+#define DEF_MENU_ENTRY_ACCELERATOR	NULL
+#define DEF_MENU_ENTRY_BG		NULL
+#define DEF_MENU_ENTRY_BITMAP		NULL
 #define DEF_MENU_ENTRY_COLUMN_BREAK	"0"
-#define DEF_MENU_ENTRY_COMMAND		((char *) NULL)
+#define DEF_MENU_ENTRY_COMMAND		NULL
 #define DEF_MENU_ENTRY_COMPOUND		"none"
-#define DEF_MENU_ENTRY_FG		((char *) NULL)
-#define DEF_MENU_ENTRY_FONT		((char *) NULL)
+#define DEF_MENU_ENTRY_FG		NULL
+#define DEF_MENU_ENTRY_FONT		NULL
 #define DEF_MENU_ENTRY_HIDE_MARGIN	"0"
-#define DEF_MENU_ENTRY_IMAGE		((char *) NULL)
+#define DEF_MENU_ENTRY_IMAGE		NULL
 #define DEF_MENU_ENTRY_INDICATOR	"1"
-#define DEF_MENU_ENTRY_LABEL		((char *) NULL)
-#define DEF_MENU_ENTRY_MENU		((char *) NULL)
+#define DEF_MENU_ENTRY_LABEL		NULL
+#define DEF_MENU_ENTRY_MENU		NULL
 #define DEF_MENU_ENTRY_OFF_VALUE	"0"
 #define DEF_MENU_ENTRY_ON_VALUE		"1"
-#define DEF_MENU_ENTRY_SELECT_IMAGE	((char *) NULL)
+#define DEF_MENU_ENTRY_SELECT_IMAGE	NULL
 #define DEF_MENU_ENTRY_STATE		"normal"
-#define DEF_MENU_ENTRY_VALUE		((char *) NULL)
-#define DEF_MENU_ENTRY_CHECK_VARIABLE	((char *) NULL)
+#define DEF_MENU_ENTRY_VALUE		NULL
+#define DEF_MENU_ENTRY_CHECK_VARIABLE	NULL
 #define DEF_MENU_ENTRY_RADIO_VARIABLE	"selectedButton"
-#define DEF_MENU_ENTRY_SELECT		((char *) NULL)
+#define DEF_MENU_ENTRY_SELECT		NULL
 #define DEF_MENU_ENTRY_UNDERLINE	"-1"
 
 /*
  * Defaults for menus overall:
  */
 
-#define DEF_MENU_ACTIVE_BG_COLOR	"systemMenuActive"
-#define DEF_MENU_ACTIVE_BG_MONO		BLACK
+#define DEF_MENU_ACTIVE_BG_COLOR	IGNORED
+#define DEF_MENU_ACTIVE_BG_MONO		IGNORED
 #define DEF_MENU_ACTIVE_BORDER_WIDTH	"0"
-#define DEF_MENU_ACTIVE_FG_COLOR	"systemMenuActiveText"
-#define DEF_MENU_ACTIVE_FG_MONO		WHITE
-#define DEF_MENU_BG_COLOR		"systemMenu"
-#define DEF_MENU_BG_MONO		WHITE
+#define DEF_MENU_ACTIVE_FG_COLOR	IGNORED
+#define DEF_MENU_ACTIVE_FG_MONO		IGNORED
+#define DEF_MENU_BG_COLOR		"#000001" /* Detects custom bg. */
+#define DEF_MENU_BG_MONO		IGNORED
 #define DEF_MENU_BORDER_WIDTH		"0"
 #define DEF_MENU_CURSOR			"arrow"
-#define DEF_MENU_DISABLED_FG_COLOR	"systemMenuDisabled"
+#define DEF_MENU_DISABLED_FG_COLOR	IGNORED
 #define DEF_MENU_DISABLED_FG_MONO	""
 #define DEF_MENU_FONT			"menu" /* special: see tkMacOSXMenu.c */
-#define DEF_MENU_FG			"systemMenuText"
+#define DEF_MENU_FG			"#010000"  /* Detects custom fg. */
 #define DEF_MENU_POST_COMMAND		""
 #define DEF_MENU_RELIEF			"flat"
-#define DEF_MENU_SELECT_COLOR		"systemMenuActive"
-#define DEF_MENU_SELECT_MONO		BLACK
+#define DEF_MENU_SELECT_COLOR		IGNORED
+#define DEF_MENU_SELECT_MONO		IGNORED
 #define DEF_MENU_TAKE_FOCUS		"0"
 #define DEF_MENU_TEAROFF		"0"
-#define DEF_MENU_TEAROFF_CMD		((char *) NULL)
+#define DEF_MENU_TEAROFF_CMD		NULL
 #define DEF_MENU_TITLE			""
 #define DEF_MENU_TYPE			"normal"
 
@@ -334,32 +325,32 @@
  * Defaults for menubuttons:
  */
 
-#define DEF_MENUBUTTON_ANCHOR		"center"
+#define DEF_MENUBUTTON_ANCHOR		"w"
 #define DEF_MENUBUTTON_ACTIVE_BG_COLOR	ACTIVE_BG
-#define DEF_MENUBUTTON_ACTIVE_BG_MONO	BLACK
+#define DEF_MENUBUTTON_ACTIVE_BG_MONO	WHITE
 #define DEF_MENUBUTTON_ACTIVE_FG_COLOR	ACTIVE_FG
-#define DEF_MENUBUTTON_ACTIVE_FG_MONO	WHITE
+#define DEF_MENUBUTTON_ACTIVE_FG_MONO	BLACK
 #define DEF_MENUBUTTON_BG_COLOR		NORMAL_BG
 #define DEF_MENUBUTTON_BG_MONO		WHITE
 #define DEF_MENUBUTTON_BITMAP		""
-#define DEF_MENUBUTTON_BORDER_WIDTH	"2"
+#define DEF_MENUBUTTON_BORDER_WIDTH	"0"
 #define DEF_MENUBUTTON_CURSOR		""
 #define DEF_MENUBUTTON_DIRECTION	"below"
 #define DEF_MENUBUTTON_DISABLED_FG_COLOR DISABLED
 #define DEF_MENUBUTTON_DISABLED_FG_MONO	""
 #define DEF_MENUBUTTON_FONT		"TkDefaultFont"
-#define DEF_MENUBUTTON_FG		BLACK
+#define DEF_MENUBUTTON_FG		NORMAL_FG
 #define DEF_MENUBUTTON_HEIGHT		"0"
 #define DEF_MENUBUTTON_HIGHLIGHT_BG_COLOR DEF_MENUBUTTON_BG_COLOR
 #define DEF_MENUBUTTON_HIGHLIGHT_BG_MONO  DEF_MENUBUTTON_BG_MONO
 #define DEF_MENUBUTTON_HIGHLIGHT	BLACK
 #define DEF_MENUBUTTON_HIGHLIGHT_WIDTH	"0"
-#define DEF_MENUBUTTON_IMAGE		((char *) NULL)
+#define DEF_MENUBUTTON_IMAGE		NULL
 #define DEF_MENUBUTTON_INDICATOR	"1"
 #define DEF_MENUBUTTON_JUSTIFY		"left"
 #define DEF_MENUBUTTON_MENU		""
-#define DEF_MENUBUTTON_PADX		"4"
-#define DEF_MENUBUTTON_PADY		"3"
+#define DEF_MENUBUTTON_PADX		"0"
+#define DEF_MENUBUTTON_PADY		"0"
 #define DEF_MENUBUTTON_RELIEF		"flat"
 #define DEF_MENUBUTTON_STATE		"normal"
 #define DEF_MENUBUTTON_TAKE_FOCUS	"0"
@@ -379,7 +370,7 @@
 #define DEF_MESSAGE_BG_MONO		WHITE
 #define DEF_MESSAGE_BORDER_WIDTH	"1"
 #define DEF_MESSAGE_CURSOR		""
-#define DEF_MESSAGE_FG			BLACK
+#define DEF_MESSAGE_FG			NORMAL_FG
 #define DEF_MESSAGE_FONT		"TkDefaultFont"
 #define DEF_MESSAGE_HIGHLIGHT_BG	NORMAL_BG
 #define DEF_MESSAGE_HIGHLIGHT		BLACK
@@ -443,7 +434,7 @@
 #define DEF_SCALE_CURSOR		""
 #define DEF_SCALE_DIGITS		"0"
 #define DEF_SCALE_FONT			"TkDefaultFont"
-#define DEF_SCALE_FG_COLOR		BLACK
+#define DEF_SCALE_FG_COLOR		NORMAL_FG
 #define DEF_SCALE_FG_MONO		BLACK
 #define DEF_SCALE_FROM			"0"
 #define DEF_SCALE_HIGHLIGHT_BG_COLOR	DEF_SCALE_BG_COLOR
@@ -463,7 +454,7 @@
 #define DEF_SCALE_SLIDER_LENGTH		"30"
 #define DEF_SCALE_SLIDER_RELIEF		"raised"
 #define DEF_SCALE_STATE			"normal"
-#define DEF_SCALE_TAKE_FOCUS		((char *) NULL)
+#define DEF_SCALE_TAKE_FOCUS		NULL
 #define DEF_SCALE_TICK_INTERVAL		"0"
 #define DEF_SCALE_TO			"100"
 #define DEF_SCALE_VARIABLE		""
@@ -490,7 +481,7 @@
 #define DEF_SCROLLBAR_RELIEF		"flat"
 #define DEF_SCROLLBAR_REPEAT_DELAY	"300"
 #define DEF_SCROLLBAR_REPEAT_INTERVAL	"100"
-#define DEF_SCROLLBAR_TAKE_FOCUS	((char *) NULL)
+#define DEF_SCROLLBAR_TAKE_FOCUS	NULL
 #define DEF_SCROLLBAR_TROUGH_COLOR	TROUGH
 #define DEF_SCROLLBAR_TROUGH_MONO	WHITE
 #define DEF_SCROLLBAR_WIDTH		"15"
@@ -500,19 +491,19 @@
  */
 
 #define DEF_TEXT_AUTO_SEPARATORS	"1"
-#define DEF_TEXT_BG_COLOR		NORMAL_BG
+#define DEF_TEXT_BG_COLOR		TEXT_BG
 #define DEF_TEXT_BG_MONO		WHITE
 #define DEF_TEXT_BLOCK_CURSOR		"0"
 #define DEF_TEXT_BORDER_WIDTH		"0"
 #define DEF_TEXT_CURSOR			"xterm"
-#define DEF_TEXT_FG			BLACK
+#define DEF_TEXT_FG			NORMAL_FG
 #define DEF_TEXT_EXPORT_SELECTION	"1"
 #define DEF_TEXT_FONT			"TkFixedFont"
 #define DEF_TEXT_HEIGHT			"24"
 #define DEF_TEXT_HIGHLIGHT_BG		NORMAL_BG
 #define DEF_TEXT_HIGHLIGHT		BLACK
 #define DEF_TEXT_HIGHLIGHT_WIDTH	"3"
-#define DEF_TEXT_INSERT_BG		BLACK
+#define DEF_TEXT_INSERT_BG		NORMAL_FG
 #define DEF_TEXT_INSERT_BD_COLOR	"0"
 #define DEF_TEXT_INSERT_BD_MONO		"0"
 #define DEF_TEXT_INSERT_OFF_TIME	"300"
@@ -538,7 +529,7 @@
 #define DEF_TEXT_STATE			"normal"
 #define DEF_TEXT_TABS			""
 #define DEF_TEXT_TABSTYLE		"tabular"
-#define DEF_TEXT_TAKE_FOCUS		((char *) NULL)
+#define DEF_TEXT_TAKE_FOCUS		NULL
 #define DEF_TEXT_UNDO			"0"
 #define DEF_TEXT_WIDTH			"80"
 #define DEF_TEXT_WRAP			"char"
@@ -551,6 +542,14 @@
 
 #define DEF_CANVTEXT_FONT		"TkDefaultFont"
 
+/*
+ * Defaults for canvas items
+ * (arcs, bitmaps, lines, polygons, rectangles, and ovals):
+ */
+
+#define DEF_CANVBMAP_FG			NORMAL_FG
+#define DEF_CANVITEM_OUTLINE		NORMAL_FG
+
 /*
  * Defaults for toplevels (most of the defaults for frames also apply
  * to toplevels):
