@@ -1,8 +1,8 @@
 /*
  * tkMacOSXFont.c --
  *
- *	Contains the Macintosh implementation of the platform-independant
- *	font package interface.
+ *	Contains the Macintosh implementation of the platform-independent font
+ *	package interface.
  *
  * Copyright 2002-2004 Benjamin Riefenstahl, Benjamin.Riefenstahl@epost.de
  * Copyright (c) 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
@@ -32,9 +32,8 @@
  */
 
 typedef struct {
-    TkFont font;		/* Stuff used by generic font package. Must
-				 * be first in structure. */
-
+    TkFont font;		/* Stuff used by generic font package. Must be
+				 * first in structure. */
     NSFont *nsFont;
     NSDictionary *nsAttributes;
 } MacFont;
@@ -57,25 +56,25 @@ struct SystemFontMapEntry {
 #define ThemeFont(n, ...) { kTheme##n##Font, "system" #n "Font", ##__VA_ARGS__ }
 static const struct SystemFontMapEntry systemFontMap[] = {
     ThemeFont(System, 			"TkDefaultFont", "TkIconFont"),
-    ThemeFont(EmphasizedSystem,		"TkCaptionFont"),
+    ThemeFont(EmphasizedSystem,		"TkCaptionFont", NULL),
     ThemeFont(SmallSystem,		"TkHeadingFont", "TkTooltipFont"),
-    ThemeFont(SmallEmphasizedSystem),
-    ThemeFont(Application,		"TkTextFont"),
-    ThemeFont(Label,			"TkSmallCaptionFont"),
-    ThemeFont(Views),
-    ThemeFont(MenuTitle),
-    ThemeFont(MenuItem,			"TkMenuFont"),
-    ThemeFont(MenuItemMark),
-    ThemeFont(MenuItemCmdKey),
-    ThemeFont(WindowTitle),
-    ThemeFont(PushButton),
-    ThemeFont(UtilityWindowTitle),
-    ThemeFont(AlertHeader),
-    ThemeFont(Toolbar),
-    ThemeFont(MiniSystem),
-    { kThemeSystemFontDetail,		"systemDetailSystemFont" },
-    { kThemeSystemFontDetailEmphasized,	"systemDetailEmphasizedSystemFont" },
-    { -1, NULL }
+    ThemeFont(SmallEmphasizedSystem, NULL, NULL),
+    ThemeFont(Application,		"TkTextFont", NULL),
+    ThemeFont(Label,			"TkSmallCaptionFont", NULL),
+    ThemeFont(Views, NULL, NULL),
+    ThemeFont(MenuTitle, NULL, NULL),
+    ThemeFont(MenuItem,			"TkMenuFont", NULL),
+    ThemeFont(MenuItemMark, NULL, NULL),
+    ThemeFont(MenuItemCmdKey, NULL, NULL),
+    ThemeFont(WindowTitle, NULL, NULL),
+    ThemeFont(PushButton, NULL, NULL),
+    ThemeFont(UtilityWindowTitle, NULL, NULL),
+    ThemeFont(AlertHeader, NULL, NULL),
+    ThemeFont(Toolbar, NULL, NULL),
+    ThemeFont(MiniSystem, NULL, NULL),
+    { kThemeSystemFontDetail,		"systemDetailSystemFont", NULL, NULL },
+    { kThemeSystemFontDetailEmphasized,	"systemDetailEmphasizedSystemFont", NULL, NULL },
+    { -1, NULL, NULL, NULL }
 };
 #undef ThemeFont
 
@@ -83,27 +82,101 @@ static int antialiasedTextEnabled = -1;
 static NSCharacterSet *whitespaceCharacterSet = nil;
 static NSCharacterSet *lineendingCharacterSet = nil;
 
-static void GetTkFontAttributesForNSFont(NSFont *nsFont,
-	TkFontAttributes *faPtr);
-static NSFont *FindNSFont(const char *familyName, NSFontTraitMask traits,
-	NSInteger weight, CGFloat size, int fallbackToDefault);
-static void InitFont(NSFont *nsFont, const TkFontAttributes *reqFaPtr,
-	MacFont * fontPtr);
-static int CreateNamedSystemFont(Tcl_Interp *interp, Tk_Window tkwin,
-	const char* name, TkFontAttributes *faPtr);
-static void DrawCharsInContext(Display *display, Drawable drawable, GC gc,
-	Tk_Font tkfont, const char *source, int numBytes, int rangeStart,
-	int rangeLength, int x, int y, double angle);
-
-@interface NSFont(TKFont)
-- (NSFont *) bestMatchingFontForCharacters: (const UTF16Char *) characters
-	length: (NSUInteger) length attributes: (NSDictionary *) attributes
-	actualCoveredLength: (NSUInteger *) coveredLength;
-@end
+static void		GetTkFontAttributesForNSFont(NSFont *nsFont,
+			    TkFontAttributes *faPtr);
+static NSFont *		FindNSFont(const char *familyName,
+			    NSFontTraitMask traits, NSInteger weight,
+			    CGFloat size, int fallbackToDefault);
+static void		InitFont(NSFont *nsFont,
+			    const TkFontAttributes *reqFaPtr,
+			    MacFont *fontPtr);
+static int		CreateNamedSystemFont(Tcl_Interp *interp,
+			    Tk_Window tkwin, const char *name,
+			    TkFontAttributes *faPtr);
 
 #pragma mark -
 #pragma mark Font Helpers:
 
+/*
+ * To avoid an extra copy, a TKNSString object wraps a Tcl_DString with an
+ * NSString that uses the DString's buffer as its character buffer.  It can be
+ * constructed from a Tcl_DString and it has a DString property that handles
+ * converting from an NSString to a Tcl_DString.
+ */
+
+@implementation TKNSString
+
+- (instancetype)initWithTclUtfBytes:(const void *)bytes
+		       length:(NSUInteger)len
+{
+    self = [self init];
+    if (self) {
+	Tcl_DStringInit(&_ds);
+	Tcl_UtfToUniCharDString(bytes, len, &_ds);
+	_string = [[NSString alloc]
+	     initWithCharactersNoCopy:(unichar *)Tcl_DStringValue(&_ds)
+			       length:Tcl_DStringLength(&_ds)>>1
+			 freeWhenDone:NO];
+	self.UTF8String = _string.UTF8String;
+    }
+    return self;
+}
+
+- (instancetype)initWithString:(NSString *)aString
+{
+    self = [self init];
+    if (self) {
+	_string = [[NSString alloc] initWithString:aString];
+	_UTF8String = _string.UTF8String;
+    }
+    return self;
+}
+
+- (void)dealloc
+{
+    Tcl_DStringFree(&_ds);
+    [_string release];
+    [super dealloc];
+}
+
+- (NSUInteger)length
+{
+    return _string.length;
+}
+
+- (unichar)characterAtIndex:(NSUInteger)index
+{
+    return [_string characterAtIndex:index];
+}
+
+- (Tcl_DString)DString
+{
+    if ( _ds.string == NULL) {
+
+	/*
+	 * The DString has not been initialized. Construct it from
+	 * our string's unicode characters.
+	 */
+	char *p;
+	NSUInteger index;
+
+	Tcl_DStringInit(&_ds);
+	Tcl_DStringSetLength(&_ds, 3 * [_string length]);
+	p = Tcl_DStringValue(&_ds);
+	for (index = 0; index < [_string length]; index++) {
+	    p += Tcl_UniCharToUtf([_string characterAtIndex: index], p);
+	}
+	Tcl_DStringSetLength(&_ds, p - Tcl_DStringValue(&_ds));
+    }
+    return _ds;
+}
+
+#ifndef __clang__
+@synthesize UTF8String = _UTF8String;
+@synthesize DString = _ds;
+#endif
+@end
+
 #define GetNSFontTraitsFromTkFontAttributes(faPtr) \
 	((faPtr)->weight == TK_FW_BOLD ? NSBoldFontMask : NSUnboldFontMask) | \
 	((faPtr)->slant == TK_FS_ITALIC ? NSItalicFontMask : NSUnitalicFontMask)
@@ -131,11 +204,11 @@ GetTkFontAttributesForNSFont(
 {
     NSFontTraitMask traits = [[NSFontManager sharedFontManager]
 	    traitsOfFont:nsFont];
-
     faPtr->family = Tk_GetUid([[nsFont familyName] UTF8String]);
     faPtr->size = [nsFont pointSize];
     faPtr->weight = (traits & NSBoldFontMask ? TK_FW_BOLD : TK_FW_NORMAL);
     faPtr->slant = (traits & NSItalicFontMask ? TK_FS_ITALIC : TK_FS_ROMAN);
+
 }
 
 /*
@@ -179,6 +252,18 @@ FindNSFont(
 	size = [defaultFont pointSize];
     }
     nsFont = [fm fontWithFamily:family traits:traits weight:weight size:size];
+
+    /*
+     * A second bug in NSFontManager that Apple created for the Catalina OS
+     * causes requests as above to sometimes return fonts with additional
+     * traits that were not requested, even though fonts without those unwanted
+     * traits exist on the system.  See bug [90d555e088].  As a workaround
+     * we ask the font manager to remove any unrequested traits.
+     */
+
+    if (nsFont) {
+	nsFont = [fm convertFont:nsFont toNotHaveTrait:~traits];
+    }
     if (!nsFont) {
 	NSArray *availableFamilies = [fm availableFontFamilies];
 	NSString *caseFamily = nil;
@@ -238,7 +323,7 @@ InitFont(
     int ascent, descent/*, dontAA*/;
     static const UniChar ch[] = {'.', 'W', ' ', 0xc4, 0xc1, 0xc2, 0xc3, 0xc7};
 			/* ., W, Space, Auml, Aacute, Acirc, Atilde, Ccedilla */
-    #define nCh (sizeof(ch) / sizeof(UniChar))
+#define nCh	(sizeof(ch) / sizeof(UniChar))
     CGGlyph glyphs[nCh];
     CGRect boundingRects[nCh];
 
@@ -250,7 +335,11 @@ InitFont(
 	TkInitFontAttributes(faPtr);
     }
     fontPtr->nsFont = nsFont;
-    // some don't like antialiasing on fixed-width even if bigger than limit
+
+    /*
+     * Some don't like antialiasing on fixed-width even if bigger than limit
+     */
+
     // dontAA = [nsFont isFixedPitch] && fontPtr->font.fa.size <= 10;
     if (antialiasedTextEnabled >= 0/* || dontAA*/) {
 	renderingMode = (antialiasedTextEnabled == 0/* || dontAA*/) ?
@@ -300,7 +389,7 @@ InitFont(
 		NSLigatureAttributeName,
 	    [NSNumber numberWithDouble:kern], NSKernAttributeName, nil];
     fontPtr->nsAttributes = [nsAttributes retain];
-    #undef nCh
+#undef nCh
 }
 
 /*
@@ -342,7 +431,7 @@ CreateNamedSystemFont(
  *
  *	This procedure is called when an application is created. It
  *	initializes all the structures that are used by the
- *	platform-dependant code on a per application basis.
+ *	platform-dependent code on a per application basis.
  *	Note that this is called before TkpInit() !
  *
  * Results:
@@ -359,15 +448,19 @@ TkpFontPkgInit(
     TkMainInfo *mainPtr)	/* The application being created. */
 {
     Tcl_Interp *interp = mainPtr->interp;
-    Tk_Window tkwin = (Tk_Window) mainPtr->winPtr;
+    Tk_Window tkwin = (Tk_Window)mainPtr->winPtr;
     const struct SystemFontMapEntry *systemFont = systemFontMap;
     NSFont *nsFont;
     TkFontAttributes fa;
     NSMutableCharacterSet *cs;
-    /* Since we called before TkpInit, we need our own autorelease pool. */
+    /*
+     * Since we called before TkpInit, we need our own autorelease pool.
+     */
     NSAutoreleasePool *pool = [NSAutoreleasePool new];
 
-    /* force this for now */
+    /*
+     * Force this for now.
+     */
     if (!mainPtr->winPtr->mainPtr) {
 	mainPtr->winPtr->mainPtr = mainPtr;
     }
@@ -389,10 +482,25 @@ TkpFontPkgInit(
 	systemFont++;
     }
     TkInitFontAttributes(&fa);
+#if 0
+
+    /*
+     * In macOS 10.15.1 Apple introduced a bug in NSFontManager which caused
+     * it to not recognize the familyName ".SF NSMono" which is the familyName
+     * of the default fixed pitch system fault on that system.  See bug [855049e799].
+     * As a workaround we call [NSFont userFixedPitchFontOfSize:11] instead.
+     * This returns a user font in the "Menlo" family.
+     */
+
     nsFont = (NSFont*) CTFontCreateUIFontForLanguage(fixedPitch, 11, NULL);
+#else
+    nsFont = [NSFont userFixedPitchFontOfSize:11];
+#endif
     if (nsFont) {
 	GetTkFontAttributesForNSFont(nsFont, &fa);
+#if 0
 	CFRelease(nsFont);
+#endif
     } else {
 	fa.family = Tk_GetUid("Monaco");
 	fa.size = 11;
@@ -419,17 +527,17 @@ TkpFontPkgInit(
  *	Map a platform-specific native font name to a TkFont.
  *
  * Results:
- *	The return value is a pointer to a TkFont that represents the
- *	native font. If a native font by the given name could not be
- *	found, the return value is NULL.
+ *	The return value is a pointer to a TkFont that represents the native
+ *	font. If a native font by the given name could not be found, the return
+ *	value is NULL.
  *
- *	Every call to this procedure returns a new TkFont structure, even
- *	if the name has already been seen before. The caller should call
+ *	Every call to this procedure returns a new TkFont structure, even if
+ *	the name has already been seen before. The caller should call
  *	TkpDeleteFont() when the font is no longer needed.
  *
- *	The caller is responsible for initializing the memory associated
- *	with the generic TkFont when this function returns and releasing
- *	the contents of the generics TkFont before calling TkpDeleteFont().
+ *	The caller is responsible for initializing the memory associated with
+ *	the generic TkFont when this function returns and releasing the
+ *	contents of the generics TkFont before calling TkpDeleteFont().
  *
  * Side effects:
  *	None.
@@ -439,7 +547,7 @@ TkpFontPkgInit(
 
 TkFont *
 TkpGetNativeFont(
-    Tk_Window tkwin,		/* For display where font will be used. */
+    TCL_UNUSED(Tk_Window),		/* For display where font will be used. */
     const char *name)		/* Platform-specific font name. */
 {
     MacFont *fontPtr = NULL;
@@ -455,10 +563,10 @@ TkpGetNativeFont(
     } else {
 	return NULL;
     }
-    ctFont = CTFontCreateUIFontForLanguage(HIThemeGetUIFontType(
-	    themeFontId), 0, NULL);
+    ctFont = CTFontCreateUIFontForLanguage(
+	    HIThemeGetUIFontType(themeFontId), 0, NULL);
     if (ctFont) {
-	fontPtr = ckalloc(sizeof(MacFont));
+	fontPtr = (MacFont *)ckalloc(sizeof(MacFont));
 	InitFont((NSFont*) ctFont, NULL, fontPtr);
     }
 
@@ -474,19 +582,18 @@ TkpGetNativeFont(
  *	closest matching attributes.
  *
  * Results:
- *	The return value is a pointer to a TkFont that represents the font
- *	with the desired attributes. If a font with the desired attributes
- *	could not be constructed, some other font will be substituted
- *	automatically.
+ *	The return value is a pointer to a TkFont that represents the font with
+ *	the desired attributes. If a font with the desired attributes could not
+ *	be constructed, some other font will be substituted automatically.
  *
- *	Every call to this procedure returns a new TkFont structure, even
- *	if the specified attributes have already been seen before. The
- *	caller should call TkpDeleteFont() to free the platform- specific
- *	data when the font is no longer needed.
+ *	Every call to this procedure returns a new TkFont structure, even if
+ *	the specified attributes have already been seen before. The caller
+ *	should call TkpDeleteFont() to free the platform- specific data when
+ *	the font is no longer needed.
  *
- *	The caller is responsible for initializing the memory associated
- *	with the generic TkFont when this function returns and releasing
- *	the contents of the generic TkFont before calling TkpDeleteFont().
+ *	The caller is responsible for initializing the memory associated with
+ *	the generic TkFont when this function returns and releasing the
+ *	contents of the generic TkFont before calling TkpDeleteFont().
  *
  * Side effects:
  *	None.
@@ -498,16 +605,16 @@ TkFont *
 TkpGetFontFromAttributes(
     TkFont *tkFontPtr,		/* If non-NULL, store the information in this
 				 * existing TkFont structure, rather than
-				 * allocating a new structure to hold the
-				 * font; the existing contents of the font
-				 * will be released. If NULL, a new TkFont
-				 * structure is allocated. */
+				 * allocating a new structure to hold the font;
+				 * the existing contents of the font will be
+				 * released. If NULL, a new TkFont structure is
+				 * allocated. */
     Tk_Window tkwin,		/* For display where font will be used. */
     const TkFontAttributes *faPtr)
 				/* Set of attributes to match. */
 {
     MacFont *fontPtr;
-    int points = (int)(TkFontGetPoints(tkwin, faPtr->size) + 0.5);
+    int points = (int) (TkFontGetPoints(tkwin, faPtr->size) + 0.5);
     NSFontTraitMask traits = GetNSFontTraitsFromTkFontAttributes(faPtr);
     NSInteger weight = (faPtr->weight == TK_FW_BOLD ? 9 : 5);
     NSFont *nsFont;
@@ -527,7 +634,7 @@ TkpGetFontFromAttributes(
 	Tcl_Panic("Could not determine NSFont from TkFontAttributes");
     }
     if (tkFontPtr == NULL) {
-	fontPtr = ckalloc(sizeof(MacFont));
+	fontPtr = (MacFont *)ckalloc(sizeof(MacFont));
     } else {
 	fontPtr = (MacFont *) tkFontPtr;
 	TkpDeleteFont(tkFontPtr);
@@ -544,9 +651,9 @@ TkpGetFontFromAttributes(
  * TkpDeleteFont --
  *
  *	Called to release a font allocated by TkpGetNativeFont() or
- *	TkpGetFontFromAttributes(). The caller should have already
- *	released the fields of the TkFont that are used exclusively by the
- *	generic TkFont code.
+ *	TkpGetFontFromAttributes(). The caller should have already released the
+ *	fields of the TkFont that are used exclusively by the generic TkFont
+ *	code.
  *
  * Results:
  *	TkFont is deallocated.
@@ -573,8 +680,8 @@ TkpDeleteFont(
  *
  * TkpGetFontFamilies --
  *
- *	Return information about the font families that are available on
- *	the display of the given window.
+ *	Return information about the font families that are available on the
+ *	display of the given window.
  *
  * Results:
  *	Modifies interp's result object to hold a list of all the available
@@ -589,7 +696,7 @@ TkpDeleteFont(
 void
 TkpGetFontFamilies(
     Tcl_Interp *interp,		/* Interp to hold result. */
-    Tk_Window tkwin)		/* For display to query. */
+    TCL_UNUSED(Tk_Window))		/* For display to query. */
 {
     Tcl_Obj *resultPtr = Tcl_NewListObj(0, NULL);
     NSArray *list = [[NSFontManager sharedFontManager] availableFontFamilies];
@@ -606,12 +713,12 @@ TkpGetFontFamilies(
  *
  * TkpGetSubFonts --
  *
- *	A function used by the testing package for querying the actual
- *	screen fonts that make up a font object.
+ *	A function used by the testing package for querying the actual screen
+ *	fonts that make up a font object.
  *
  * Results:
- *	Modifies interp's result object to hold a list containing the names
- *	of the screen fonts that make up the given font object.
+ *	Modifies interp's result object to hold a list containing the names of
+ *	the screen fonts that make up the given font object.
  *
  * Side effects:
  *	None.
@@ -648,8 +755,8 @@ TkpGetSubFonts(
  *
  * TkpGetFontAttrsForChar --
  *
- *	Retrieve the font attributes of the actual font used to render a
- *	given character.
+ *	Retrieve the font attributes of the actual font used to render a given
+ *	character.
  *
  * Results:
  *	None.
@@ -662,7 +769,7 @@ TkpGetSubFonts(
 
 void
 TkpGetFontAttrsForChar(
-    Tk_Window tkwin,		/* Window on the font's display */
+    TCL_UNUSED(Tk_Window),		/* Window on the font's display */
     Tk_Font tkfont,		/* Font to query */
     int c,         		/* Character of interest */
     TkFontAttributes* faPtr)	/* Output: Font attributes */
@@ -751,10 +858,9 @@ Tk_MeasureChars(
  *	all the characters on the line for context.
  *
  * Results:
- *	The return value is the number of bytes from source that
- *	fit into the span that extends from 0 to maxLength. *lengthPtr is
- *	filled with the x-coordinate of the right edge of the last
- *	character that did fit.
+ *	The return value is the number of bytes from source that fit into the
+ *	span that extends from 0 to maxLength. *lengthPtr is filled with the
+ *	x-coordinate of the right edge of the last character that did fit.
  *
  * Side effects:
  *	None.
@@ -780,11 +886,11 @@ TkpMeasureCharsInContext(
 				 * TK_PARTIAL_OK means include the last char
 				 * which only partially fits on this line.
 				 * TK_WHOLE_WORDS means stop on a word
-				 * boundary, if possible. TK_AT_LEAST_ONE
-				 * means return at least one character even
-				 * if no characters fit.  If TK_WHOLE_WORDS
-				 * and TK_AT_LEAST_ONE are set and the first
-				 * word doesn't fit, we return at least one
+				 * boundary, if possible. TK_AT_LEAST_ONE means
+				 * return at least one character even if no
+				 * characters fit.  If TK_WHOLE_WORDS and
+				 * TK_AT_LEAST_ONE are set and the first word
+				 * doesn't fit, we return at least one
 				 * character or whatever characters fit into
 				 * maxLength.  TK_ISOLATE_END means that the
 				 * last character should not be considered in
@@ -814,8 +920,7 @@ TkpMeasureCharsInContext(
     if (maxLength > 32767) {
 	maxLength = 32767;
     }
-    string = [[NSString alloc] initWithBytesNoCopy:(void*)source
-		length:numBytes encoding:NSUTF8StringEncoding freeWhenDone:NO];
+    string = [[TKNSString alloc] initWithTclUtfBytes:source length:numBytes];
     if (!string) {
 	length = 0;
 	fit = rangeLength;
@@ -870,14 +975,13 @@ TkpMeasureCharsInContext(
 	}
 
         /*
-         * If there is no line breakpoint in the source string between
-         * its start and the index position that fits in maxWidth, then
+         * If there is no line breakpoint in the source string between its
+         * start and the index position that fits in maxWidth, then
          * CTTypesetterSuggestLineBreak() returns that very last index.
-         * However if the TK_WHOLE_WORDS flag is set, we want to break
-         * at a word boundary. In this situation, unless TK_AT_LEAST_ONE
-         * is set, we must report that zero chars actually fit (in other
-         * words the smallest word of the source string is still larger
-         * than maxWidth).
+         * However if the TK_WHOLE_WORDS flag is set, we want to break at a
+         * word boundary. In this situation, unless TK_AT_LEAST_ONE is set, we
+         * must report that zero chars actually fit (in other words the
+         * smallest word of the source string is still larger than maxWidth).
          */
 
         if ((index >= start) && (index < len) &&
@@ -909,9 +1013,12 @@ TkpMeasureCharsInContext(
 	    CFRelease(line);
 	}
 
-        /* The call to CTTypesetterSuggestClusterBreak above will always
-           return at least one character regardless of whether it exceeded
-           it or not.  Clean that up now. */
+        /*
+	 * The call to CTTypesetterSuggestClusterBreak above will always return
+	 * at least one character regardless of whether it exceeded it or not.
+	 * Clean that up now.
+	 */
+
 	while (width > maxWidth && !(flags & TK_PARTIAL_OK)
 		&& index > start+(flags & TK_AT_LEAST_ONE)) {
 	    range.length = --index;
@@ -925,7 +1032,7 @@ TkpMeasureCharsInContext(
     [attributedString release];
     [string release];
     length = ceil(width - offset);
-    fit = (Tcl_UtfAtIndex(source, index) - source) - rangeStart;
+    fit = (TkUtfAtIndex(source, index) - source) - rangeStart;
 done:
 #ifdef TK_MAC_DEBUG_FONTS
     TkMacOSXDbgMsg("measure: source=\"%s\" range=\"%.*s\" maxLength=%d "
@@ -949,10 +1056,10 @@ done:
  *	Draw a string of characters on the screen.
  *
  *	With ATSUI we need the line context to do this right, so we have the
- *	actual implementation in TkpDrawCharsInContext().
+ *	actual implementation in TkpDrawAngledCharsInContext().
  *
  * Results:
-  *	None.
+ *	None.
  *
  * Side effects:
  *	Information gets drawn on the screen.
@@ -970,15 +1077,15 @@ Tk_DrawChars(
     const char *source,		/* UTF-8 string to be displayed. Need not be
 				 * '\0' terminated. All Tk meta-characters
 				 * (tabs, control characters, and newlines)
-				 * should be stripped out of the string that
-				 * is passed to this function. If they are not
+				 * should be stripped out of the string that is
+				 * passed to this function. If they are not
 				 * stripped out, they will be displayed as
 				 * regular printing characters. */
     int numBytes,		/* Number of bytes in string. */
     int x, int y)		/* Coordinates at which to place origin of the
 				 * string when drawing. */
 {
-    DrawCharsInContext(display, drawable, gc, tkfont, source, numBytes,
+    TkpDrawAngledCharsInContext(display, drawable, gc, tkfont, source, numBytes,
 	    0, numBytes, x, y, 0.0);
 }
 
@@ -992,8 +1099,8 @@ TkDrawAngledChars(
     const char *source,		/* UTF-8 string to be displayed. Need not be
 				 * '\0' terminated. All Tk meta-characters
 				 * (tabs, control characters, and newlines)
-				 * should be stripped out of the string that
-				 * is passed to this function. If they are not
+				 * should be stripped out of the string that is
+				 * passed to this function. If they are not
 				 * stripped out, they will be displayed as
 				 * regular printing characters. */
     int numBytes,		/* Number of bytes in string. */
@@ -1001,7 +1108,7 @@ TkDrawAngledChars(
 				 * string when drawing. */
     double angle)		/* What angle to put text at, in degrees. */
 {
-    DrawCharsInContext(display, drawable, gc, tkfont, source, numBytes,
+    TkpDrawAngledCharsInContext(display, drawable, gc, tkfont, source, numBytes,
 	    0, numBytes, x, y, angle);
 }
 
@@ -1035,8 +1142,8 @@ TkpDrawCharsInContext(
     const char * source,	/* UTF-8 string to be displayed. Need not be
 				 * '\0' terminated. All Tk meta-characters
 				 * (tabs, control characters, and newlines)
-				 * should be stripped out of the string that
-				 * is passed to this function. If they are not
+				 * should be stripped out of the string that is
+				 * passed to this function. If they are not
 				 * stripped out, they will be displayed as
 				 * regular printing characters. */
     int numBytes,		/* Number of bytes in string. */
@@ -1046,13 +1153,13 @@ TkpDrawCharsInContext(
 				 * whole (not just the range) string when
 				 * drawing. */
 {
-    DrawCharsInContext(display, drawable, gc, tkfont, source, numBytes,
+    TkpDrawAngledCharsInContext(display, drawable, gc, tkfont, source, numBytes,
 	    rangeStart, rangeLength, x, y, 0.0);
 }
 
-static void
-DrawCharsInContext(
-    Display *display,		/* Display on which to draw. */
+void
+TkpDrawAngledCharsInContext(
+    TCL_UNUSED(Display *),		/* Display on which to draw. */
     Drawable drawable,		/* Window or pixmap in which to draw. */
     GC gc,			/* Graphics context for drawing characters. */
     Tk_Font tkfont,		/* Font in which characters will be drawn; must
@@ -1060,78 +1167,84 @@ DrawCharsInContext(
     const char * source,	/* UTF-8 string to be displayed. Need not be
 				 * '\0' terminated. All Tk meta-characters
 				 * (tabs, control characters, and newlines)
-				 * should be stripped out of the string that
-				 * is passed to this function. If they are not
+				 * should be stripped out of the string that is
+				 * passed to this function. If they are not
 				 * stripped out, they will be displayed as
 				 * regular printing characters. */
     int numBytes,		/* Number of bytes in string. */
     int rangeStart,		/* Index of first byte to draw. */
     int rangeLength,		/* Length of range to draw in bytes. */
-    int x, int y,		/* Coordinates at which to place origin of the
+    double x, double y,		/* Coordinates at which to place origin of the
 				 * whole (not just the range) string when
 				 * drawing. */
-    double angle)
+    double angle)		/* What angle to put text at, in degrees. */
 {
     const MacFont *fontPtr = (const MacFont *) tkfont;
     NSString *string;
     NSMutableDictionary *attributes;
     NSAttributedString *attributedString;
     CTTypesetterRef typesetter;
-    CFIndex start, len;
-    CTLineRef line;
-    MacDrawable *macWin = (MacDrawable *) drawable;
+    CFIndex start, length;
+    CTLineRef line, full=nil;
+    MacDrawable *macWin = (MacDrawable *)drawable;
     TkMacOSXDrawingContext drawingContext;
     CGContextRef context;
     CGColorRef fg;
     NSFont *nsFont;
     CGAffineTransform t;
-    int h;
+    CGFloat width, height, textX = (CGFloat) x, textY = (CGFloat) y;
 
-    if (rangeStart < 0 || rangeLength <= 0 ||
-	    rangeStart + rangeLength > numBytes ||
-	    !TkMacOSXSetupDrawingContext(drawable, gc, 1, &drawingContext)) {
+    if (rangeStart < 0 || rangeLength <= 0  ||
+	rangeStart + rangeLength > numBytes ||
+	!TkMacOSXSetupDrawingContext(drawable, gc, &drawingContext)) {
 	return;
     }
-    string = [[NSString alloc] initWithBytesNoCopy:(void*)source
-		length:numBytes encoding:NSUTF8StringEncoding freeWhenDone:NO];
+    string = [[TKNSString alloc] initWithTclUtfBytes:source length:numBytes];
     if (!string) {
 	return;
     }
+
     context = drawingContext.context;
-    fg = TkMacOSXCreateCGColor(gc, gc->foreground);
+    TkSetMacColor(gc->foreground, &fg);
     attributes = [fontPtr->nsAttributes mutableCopy];
     [attributes setObject:(id)fg forKey:(id)kCTForegroundColorAttributeName];
-    CFRelease(fg);
     nsFont = [attributes objectForKey:NSFontAttributeName];
-    [nsFont setInContext:[NSGraphicsContext graphicsContextWithGraphicsPort:
-	    context flipped:NO]];
+    [nsFont setInContext:GET_NSCONTEXT(context, NO)];
     CGContextSetTextMatrix(context, CGAffineTransformIdentity);
     attributedString = [[NSAttributedString alloc] initWithString:string
 	    attributes:attributes];
     typesetter = CTTypesetterCreateWithAttributedString(
 	    (CFAttributedStringRef)attributedString);
-    x += macWin->xOff;
-    y += macWin->yOff;
-    h = drawingContext.portBounds.size.height;
-    y = h - y;
-    t = CGAffineTransformMake(1.0, 0.0, 0.0, -1.0, 0.0, h);
+    textX += (CGFloat) macWin->xOff;
+    textY += (CGFloat) macWin->yOff;
+    height = drawingContext.portBounds.size.height;
+    textY = height - textY;
+    t = CGAffineTransformMake(1.0, 0.0, 0.0, -1.0, 0.0, height);
     if (angle != 0.0) {
-	t = CGAffineTransformTranslate(CGAffineTransformRotate(
-		CGAffineTransformTranslate(t, x, y), angle*PI/180.0), -x, -y);
+	t = CGAffineTransformTranslate(
+             CGAffineTransformRotate(
+                 CGAffineTransformTranslate(t, textX, textY), angle*PI/180.0),
+             -textX, -textY);
     }
     CGContextConcatCTM(context, t);
-    CGContextSetTextPosition(context, x, y);
     start = Tcl_NumUtfChars(source, rangeStart);
-    len = Tcl_NumUtfChars(source, rangeStart + rangeLength);
+    length = Tcl_NumUtfChars(source, rangeStart + rangeLength) - start;
+    line = CTTypesetterCreateLine(typesetter, CFRangeMake(start, length));
     if (start > 0) {
-	CGRect clipRect = CGRectInfinite, startBounds;
-	line = CTTypesetterCreateLine(typesetter, CFRangeMake(0, start));
-	startBounds = CTLineGetImageBounds(line, context);
-	CFRelease(line);
-	clipRect.origin.x = startBounds.origin.x + startBounds.size.width;
-	CGContextClipToRect(context, clipRect);
+
+	/*
+	 * We are only drawing part of the string.  To compute the x coordinate
+	 * of the part we are drawing we subtract its typographical length from
+	 * the typographical length of the full string.  This accounts for the
+	 * kerning after the initial part of the string.
+	 */
+
+	full = CTTypesetterCreateLine(typesetter, CFRangeMake(0, start + length));
+	width = CTLineGetTypographicBounds(full, NULL, NULL, NULL);
+	CFRelease(full);
+	textX += (width - CTLineGetTypographicBounds(line, NULL, NULL, NULL));
     }
-    line = CTTypesetterCreateLine(typesetter, CFRangeMake(0, len));
+    CGContextSetTextPosition(context, textX, textY);
     CTLineDraw(line, context);
     CFRelease(line);
     CFRelease(typesetter);
@@ -1209,8 +1322,8 @@ TkMacOSXNSFontAttributesForFont(
 
 int
 TkMacOSXIsCharacterMissing(
-    Tk_Font tkfont,		/* The font we are looking in. */
-    unsigned int searchChar)	/* The character we are looking for. */
+    TCL_UNUSED(Tk_Font),		/* The font we are looking in. */
+    TCL_UNUSED(unsigned int))	/* The character we are looking for. */
 {
     return 0;
 }
@@ -1247,9 +1360,10 @@ TkMacOSXFontDescriptionForNSFontAndNSFontAttributes(
 		NSUnderlineStyleAttributeName];
 	id strikethrough = [nsAttributes objectForKey:
 		NSStrikethroughStyleAttributeName];
+
 	objv[i++] = Tcl_NewStringObj(familyName, -1);
-	objv[i++] = Tcl_NewIntObj([nsFont pointSize]);
-#define S(s) Tcl_NewStringObj(STRINGIFY(s),(int)(sizeof(STRINGIFY(s))-1))
+	objv[i++] = Tcl_NewWideIntObj([nsFont pointSize]);
+#define S(s)    Tcl_NewStringObj(STRINGIFY(s), (sizeof(STRINGIFY(s))-1))
 	objv[i++] = (traits & NSBoldFontMask)	? S(bold)   : S(normal);
 	objv[i++] = (traits & NSItalicFontMask)	? S(italic) : S(roman);
 	if ([underline respondsToSelector:@selector(intValue)] &&
@@ -1273,8 +1387,9 @@ TkMacOSXFontDescriptionForNSFontAndNSFontAttributes(
  * TkMacOSXUseAntialiasedText --
  *
  *	Enables or disables application-wide use of antialiased text (where
- *	available). Sets up a linked Tcl global variable to allow
- *	disabling of antialiased text from tcl.
+ *	available). Sets up a linked Tcl global variable to allow disabling of
+ *	antialiased text from Tcl.
+ *
  *	The possible values for this variable are:
  *
  *	-1 - Use system default as configurable in "System Prefs" -> "General".
