@@ -25,128 +25,246 @@ typedef struct {
     Point local;
 } MouseEventData;
 
+static Tk_Window captureWinPtr = NULL;	/* Current capture window; may be
+					 * NULL. */
+
 static int		GenerateButtonEvent(MouseEventData *medPtr);
 static unsigned int	ButtonModifiers2State(UInt32 buttonState,
-					      UInt32 keyModifiers);
+			    UInt32 keyModifiers);
 
 #pragma mark TKApplication(TKMouseEvent)
 
 enum {
     NSWindowWillMoveEventType = 20
 };
+
 /*
  * In OS X 10.6 an NSEvent of type NSMouseMoved would always have a non-Nil
  * window attribute pointing to the active window.  As of 10.8 this behavior
  * had changed.  The new behavior was that if the mouse were ever moved outside
  * of a window, all subsequent NSMouseMoved NSEvents would have a Nil window
- * attribute.  To work around this the TKApplication remembers the last non-Nil
- * window that it received in a mouse event. If it receives an NSEvent with a
- * Nil window attribute then the saved window is used.
+ * attribute until the mouse returned to the window.  In 11.1 it changed again.
+ * The window attribute can be non-nil, but referencing a window which does not
+ * belong to the application.
  */
 
 @implementation TKApplication(TKMouseEvent)
 - (NSEvent *) tkProcessMouseEvent: (NSEvent *) theEvent
 {
+    NSWindow *eventWindow = [theEvent window];
+    NSEventType eventType = [theEvent type];
+    NSRect viewFrame = [[eventWindow contentView] frame];
+    NSPoint location = [theEvent locationInWindow];
+    TkWindow *winPtr = NULL, *grabWinPtr;
+    Tk_Window tkwin = None, capture, target;
+    NSPoint local, global;
+    NSInteger button;
+    Bool inTitleBar = NO;
+    int win_x, win_y;
+    unsigned int buttonState = 0;
+    static int validPresses = 0, ignoredPresses = 0;
+
 #ifdef TK_MAC_DEBUG_EVENTS
     TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, theEvent);
 #endif
-    NSWindow*    eventWindow = [theEvent window];
-    NSEventType	 eventType = [theEvent type];
-#if 0
-    NSTrackingArea  *trackingArea = nil;
-    NSInteger eventNumber, clickCount, buttonNumber;
-#endif
+
+    /*
+     * If this event is not for a Tk toplevel, it should just be passed up the
+     * responder chain.  However, there is an exception for synthesized events,
+     * which are used in testing.  Those events are recognized by having their
+     * both the windowNumber and the eventNumber set to -1.
+     */
+
+    if (eventWindow && ![eventWindow isMemberOfClass:[TKWindow class]]) {
+	if ([theEvent windowNumber] != -1 || [theEvent eventNumber] != -1)
+	    return theEvent;
+    }
+
+    /*
+     * Check if the event is located in the titlebar.
+     */
+
+    if (eventWindow) {
+	inTitleBar = viewFrame.size.height < location.y;
+    }
+
+    button = [theEvent buttonNumber] + Button1;
     switch (eventType) {
-    case NSMouseEntered:
-    case NSMouseExited:
-    case NSCursorUpdate:
-    case NSLeftMouseDown:
-    case NSLeftMouseUp:
-    case NSRightMouseDown:
     case NSRightMouseUp:
-    case NSOtherMouseDown:
     case NSOtherMouseUp:
+	buttonState &= ~TkGetButtonMask(button);
+	break;
     case NSLeftMouseDragged:
     case NSRightMouseDragged:
     case NSOtherMouseDragged:
+    case NSRightMouseDown:
+    case NSOtherMouseDown:
+	buttonState |= TkGetButtonMask(button);
+	break;
+    case NSMouseEntered:
+	if ([eventWindow respondsToSelector:@selector(mouseInResizeArea)] &&
+	    !inTitleBar) {
+	    [(TKWindow *)eventWindow setMouseInResizeArea:YES];
+	}
+	break;
+    case NSMouseExited:
+	if ([eventWindow respondsToSelector:@selector(mouseInResizeArea)]) {
+	    [(TKWindow *)eventWindow setMouseInResizeArea:NO];
+	    break;
+	}
+    case NSLeftMouseUp:
+    case NSLeftMouseDown:
     case NSMouseMoved:
+    case NSScrollWheel:
+#if 0
+    case NSCursorUpdate:
     case NSTabletPoint:
     case NSTabletProximity:
-    case NSScrollWheel:
-        break;
-    default: /* Unrecognized mouse event. */
+#endif
+	break;
+    default: /* This type of event is ignored. */
 	return theEvent;
     }
 
-    /* Remember the window in case we need it next time. */
-    if (eventWindow && eventWindow != _windowWithMouse) {
-	if (_windowWithMouse) {
-	    [_windowWithMouse release];
-	}
-	_windowWithMouse = eventWindow;
-	[_windowWithMouse retain];
-    }
+    /*
+     * Update the button state.  We ignore left button presses that start a
+     * resize or occur in the title bar.  See tickets [d72abe6b54] and
+     * [39cbacb9e8].
+     */
 
-    /* Create an Xevent to add to the Tk queue. */
-    NSPoint global, local = [theEvent locationInWindow];
-    if (eventWindow) { /* local will be in window coordinates. */
-	global = [eventWindow tkConvertPointToScreen: local];
-	local.y = [eventWindow frame].size.height - local.y;
-	global.y = tkMacOSXZeroScreenHeight - global.y;
-    } else { /* local will be in screen coordinates. */
-	if (_windowWithMouse ) {
-	    eventWindow = _windowWithMouse;
-	    global = local;
-	    local = [eventWindow tkConvertPointFromScreen: local];
-	    local.y = [eventWindow frame].size.height - local.y;
-	    global.y = tkMacOSXZeroScreenHeight - global.y;
-	} else { /* We have no window. Use the screen???*/
-	    local.y = tkMacOSXZeroScreenHeight - local.y;
-	    global = local;
+    if (eventType == NSLeftMouseDown) {
+	if ([eventWindow respondsToSelector:@selector(mouseInResizeArea)] &&
+	    [(TKWindow *) eventWindow mouseInResizeArea]) {
+
+	    /*
+	     * When the left button is pressed in the resize area, we receive
+	     * NSMouseDown, but when it is released we do not receive
+	     * NSMouseUp.  So ignore the event and clear the button state but
+	     * do not change the ignoredPresses count.
+	     */
+
+	    buttonState &= ~TkGetButtonMask(Button1);
+	    return theEvent;
+	}
+	if (inTitleBar) {
+	    ignoredPresses++;
+	    return theEvent;
+	}
+	validPresses++;
+	buttonState |= TkGetButtonMask(Button1);
+    }
+    if (eventType == NSLeftMouseUp) {
+	if (ignoredPresses > 0) {
+	    ignoredPresses--;
+	} else if (validPresses > 0) {
+	    validPresses--;
+	}
+	if (validPresses == 0) {
+	    buttonState &= ~TkGetButtonMask(Button1);
 	}
     }
 
-    Window window = TkMacOSXGetXWindow(eventWindow);
-    Tk_Window tkwin = window ? Tk_IdToWindow(TkGetDisplayList()->display,
-	    window) : NULL;
-    if (!tkwin) {
-	tkwin = TkMacOSXGetCapture();
+    /*
+     * Find an appropriate NSWindow to attach to this event, and its
+     * associated Tk window.
+     */
+
+    capture = TkMacOSXGetCapture();
+    if (capture) {
+	winPtr = (TkWindow *) capture;
+	eventWindow = TkMacOSXGetNSWindowForDrawable(winPtr->window);
+	if (!eventWindow) {
+	    return theEvent;
+	}
+    } else {
+	if (eventWindow) {
+	    winPtr = TkMacOSXGetTkWindow(eventWindow);
+	}
+	if (!winPtr) {
+	    eventWindow = [NSApp mainWindow];
+	    winPtr = TkMacOSXGetTkWindow(eventWindow);
+	}
     }
-    if (!tkwin) {
-	return theEvent; /* Give up.  No window for this event. */
+    if (!winPtr) {
+
+	/*
+	 * We couldn't find a Tk window for this event.  We have to ignore it.
+	 */
+
+#ifdef TK_MAC_DEBUG_EVENTS
+	TkMacOSXDbgMsg("Event received with no Tk window.");
+#endif
+	return theEvent;
+    }
+    tkwin = (Tk_Window) winPtr;
+
+    /*
+     * Compute the mouse position in local (window) and global (screen)
+     * coordinates.  These are Tk coordinates, meaning that the local origin is
+     * at the top left corner of the containing toplevel and the global origin
+     * is at top left corner of the primary screen.
+     */
+
+    global = [NSEvent mouseLocation];
+    local = [eventWindow tkConvertPointFromScreen: global];
+    global.x = floor(global.x);
+    global.y = floor(TkMacOSXZeroScreenHeight() - global.y);
+    local.x = floor(local.x);
+    local.y = floor([eventWindow frame].size.height - local.y);
+    if (Tk_IsEmbedded(winPtr)) {
+	TkWindow *contPtr = TkpGetOtherWindow(winPtr);
+	if (Tk_IsTopLevel(contPtr)) {
+	    local.x -= contPtr->wmInfoPtr->xInParent;
+	    local.y -= contPtr->wmInfoPtr->yInParent;
+	} else {
+	    TkWindow *topPtr = TkMacOSXGetHostToplevel(winPtr)->winPtr;
+	    local.x -= (topPtr->wmInfoPtr->xInParent + contPtr->changes.x);
+	    local.y -= (topPtr->wmInfoPtr->yInParent + contPtr->changes.y);
+	}
+    } else {
+	local.x -= winPtr->wmInfoPtr->xInParent;
+	local.y -= winPtr->wmInfoPtr->yInParent;
     }
 
-    TkWindow  *winPtr = (TkWindow *) tkwin;
-    local.x -= winPtr->wmInfoPtr->xInParent;
-    local.y -= winPtr->wmInfoPtr->yInParent;
+    /*
+     * Use the local coordinates to find the Tk window which should receive
+     * this event.  Also convert local into the coordinates of that window.
+     * (The converted local coordinates are only needed for scrollwheel
+     * events.)
+     */
 
-    int win_x, win_y;
-    tkwin = Tk_TopCoordsToWindow(tkwin, local.x, local.y,
-		&win_x, &win_y);
+    target = Tk_TopCoordsToWindow(tkwin, local.x, local.y, &win_x, &win_y);
 
-    unsigned int state = 0;
-    NSInteger button = [theEvent buttonNumber];
-    EventRef eventRef = (EventRef)[theEvent eventRef];
-    UInt32 buttons;
-    OSStatus err = GetEventParameter(eventRef, kEventParamMouseChord,
-				     typeUInt32, NULL, sizeof(UInt32), NULL, &buttons);
+    /*
+     * Ignore the event if a local grab is in effect and the Tk window is
+     * not in the grabber's subtree.
+     */
 
-    if (err == noErr) {
-    	state |= (buttons & ((1<<5) - 1)) << 8;
-    } else if (button < 5) {
-	switch (eventType) {
-	case NSLeftMouseDown:
-	case NSRightMouseDown:
-	case NSLeftMouseDragged:
-	case NSRightMouseDragged:
-	case NSOtherMouseDown:
-	    state |= 1 << (button + 8);
-	    break;
-	default:
-	    break;
+    grabWinPtr = winPtr->dispPtr->grabWinPtr;
+    if (grabWinPtr && /* There is a grab in effect ... */
+	!winPtr->dispPtr->grabFlags && /* and it is a local grab ... */
+	grabWinPtr->mainPtr == winPtr->mainPtr){ /* in the same application. */
+	Tk_Window tkwin2;
+	if (!target) {
+	    return theEvent;
+	}
+	for (tkwin2 = target;
+	     !Tk_IsTopLevel(tkwin2);
+	     tkwin2 = Tk_Parent(tkwin2)) {
+	    if (tkwin2 == (Tk_Window)grabWinPtr) {
+		break;
+	    }
+	}
+	if (tkwin2 != (Tk_Window)grabWinPtr) {
+	    return theEvent;
 	}
     }
 
+    /*
+     *  Generate an XEvent for this mouse event.
+     */
+
+    unsigned int state = buttonState;
     NSUInteger modifiers = [theEvent modifierFlags];
 
     if (modifiers & NSAlphaShiftKeyMask) {
@@ -172,28 +290,41 @@ enum {
     }
 
     if (eventType != NSScrollWheel) {
+
+	/*
+	 * For normal mouse events, Tk_UpdatePointer will send the appropriate
+	 * XEvents using its cached state information.  Unfortunately, it will
+	 * also recompute the local coordinates.
+	 */
+
 #ifdef TK_MAC_DEBUG_EVENTS
-	TKLog(@"UpdatePointer %p x %f.0 y %f.0 %d", tkwin, global.x, global.y, state);
+	TKLog(@"UpdatePointer %p x %.1f y %.1f %d",
+		target, global.x, global.y, state);
 #endif
-	Tk_UpdatePointer(tkwin, global.x, global.y, state);
-    } else { /* handle scroll wheel event */
+
+	Tk_UpdatePointer(target, global.x, global.y, state);
+    } else {
 	CGFloat delta;
 	int coarseDelta;
 	XEvent xEvent;
 
+	/*
+	 * For scroll wheel events we need to send the XEvent here.
+	 */
+
 	xEvent.type = MouseWheelEvent;
-	xEvent.xbutton.x = local.x;
-	xEvent.xbutton.y = local.y;
+	xEvent.xbutton.x = win_x;
+	xEvent.xbutton.y = win_y;
 	xEvent.xbutton.x_root = global.x;
 	xEvent.xbutton.y_root = global.y;
 	xEvent.xany.send_event = false;
-	xEvent.xany.display = Tk_Display(tkwin);
-	xEvent.xany.window = Tk_WindowId(tkwin);
+	xEvent.xany.display = Tk_Display(target);
+	xEvent.xany.window = Tk_WindowId(target);
 
 	delta = [theEvent deltaY];
 	if (delta != 0.0) {
 	    coarseDelta = (delta > -1.0 && delta < 1.0) ?
-		(signbit(delta) ? -1 : 1) : lround(delta);
+		    (signbit(delta) ? -1 : 1) : lround(delta);
 	    xEvent.xbutton.state = state;
 	    xEvent.xkey.keycode = coarseDelta;
 	    xEvent.xany.serial = LastKnownRequestProcessed(Tk_Display(tkwin));
@@ -202,7 +333,7 @@ enum {
 	delta = [theEvent deltaX];
 	if (delta != 0.0) {
 	    coarseDelta = (delta > -1.0 && delta < 1.0) ?
-		(signbit(delta) ? -1 : 1) : lround(delta);
+		    (signbit(delta) ? -1 : 1) : lround(delta);
 	    xEvent.xbutton.state = state | ShiftMask;
 	    xEvent.xkey.keycode = coarseDelta;
 	    xEvent.xany.serial = LastKnownRequestProcessed(Tk_Display(tkwin));
@@ -215,34 +346,6 @@ enum {
 
 #pragma mark -
 
-/*
- *----------------------------------------------------------------------
- *
- * TkMacOSXKeyModifiers --
- *
- *	Returns the current state of the modifier keys.
- *
- * Results:
- *	An OS Modifier state.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-EventModifiers
-TkMacOSXModifierState(void)
-{
-    UInt32 keyModifiers;
-    int isFrontProcess = (GetCurrentEvent() && Tk_MacOSXIsAppInFront());
-
-    keyModifiers = isFrontProcess ? GetCurrentEventKeyModifiers() :
-	    GetCurrentKeyModifiers();
-
-    return (EventModifiers) (keyModifiers & USHRT_MAX);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -299,10 +402,10 @@ ButtonModifiers2State(
     unsigned int state;
 
     /*
-     * Tk supports at most 5 buttons.
+     * Tk on OSX supports at most 5 buttons.
      */
 
-    state = (buttonState & ((1<<5) - 1)) << 8;
+    state = (buttonState & 0x1F) * Button1Mask;
 
     if (keyModifiers & alphaLock) {
 	state |= LockMask;
@@ -367,8 +470,8 @@ XQueryPointer(
 	NSPoint global = [NSEvent mouseLocation];
 
 	if (getLocal) {
-	    MacDrawable *macWin = (MacDrawable *) w;
-	    NSWindow *win = TkMacOSXDrawableWindow(w);
+	    MacDrawable *macWin = (MacDrawable *)w;
+	    NSWindow *win = TkMacOSXGetNSWindowForDrawable(w);
 
 	    if (win) {
 		NSPoint local;
@@ -385,7 +488,7 @@ XQueryPointer(
 	}
 	if (getGlobal) {
 	    *root_x_return = global.x;
-	    *root_y_return = tkMacOSXZeroScreenHeight - global.y;
+	    *root_y_return = TkMacOSXZeroScreenHeight() - global.y;
 	}
     }
     if (mask_return) {
@@ -406,7 +509,7 @@ XQueryPointer(
  *	True if event(s) are generated - false otherwise.
  *
  * Side effects:
- *	Additional events may be place on the Tk event queue. Grab state may
+ *	Additional events may be placed on the Tk event queue. Grab state may
  *	also change.
  *
  *----------------------------------------------------------------------
@@ -437,14 +540,14 @@ TkGenerateButtonEventForXPointer(
  * TkGenerateButtonEvent --
  *
  *	Given a global x & y position and the button key status this procedure
- *	generates the appropiate X button event. It also handles the state
+ *	generates the appropriate X button event. It also handles the state
  *	changes needed to implement implicit grabs.
  *
  * Results:
  *	True if event(s) are generated, false otherwise.
  *
  * Side effects:
- *	Additional events may be place on the Tk event queue. Grab state may
+ *	Additional events may be placed on the Tk event queue. Grab state may
  *	also change.
  *
  *----------------------------------------------------------------------
@@ -457,8 +560,8 @@ TkGenerateButtonEvent(
     Window window,		/* X Window containing button event. */
     unsigned int state)		/* Button Key state suitable for X event. */
 {
-    MacDrawable *macWin = (MacDrawable *) window;
-    NSWindow *win = TkMacOSXDrawableWindow(window);
+    MacDrawable *macWin = (MacDrawable *)window;
+    NSWindow *win = TkMacOSXGetNSWindowForDrawable(window);
     MouseEventData med;
 
     bzero(&med, sizeof(MouseEventData));
@@ -469,7 +572,7 @@ TkGenerateButtonEvent(
     med.local = med.global;
 
     if (win) {
-	NSPoint local = NSMakePoint(x, tkMacOSXZeroScreenHeight - y);
+	NSPoint local = NSMakePoint(x, TkMacOSXZeroScreenHeight() - y);
 
 	local = [win tkConvertPointFromScreen:local];
 	local.y = [win frame].size.height - local.y;
@@ -478,7 +581,7 @@ TkGenerateButtonEvent(
 	    local.y -= macWin->winPtr->wmInfoPtr->yInParent;
 	}
 	med.local.h = local.x;
-	med.local.v = tkMacOSXZeroScreenHeight - local.y;
+	med.local.v = TkMacOSXZeroScreenHeight() - local.y;
     }
 
     return GenerateButtonEvent(&med);
@@ -496,7 +599,7 @@ TkGenerateButtonEvent(
  *	True if event(s) are generated - false otherwise.
  *
  * Side effects:
- *	Additional events may be place on the Tk event queue. Grab state may
+ *	Additional events may be placed on the Tk event queue. Grab state may
  *	also change.
  *
  *----------------------------------------------------------------------
@@ -511,6 +614,7 @@ GenerateButtonEvent(
     TkDisplay *dispPtr;
 
 #if UNUSED
+
     /*
      * ButtonDown events will always occur in the front window. ButtonUp
      * events, however, may occur anywhere on the screen. ButtonUp events
@@ -538,16 +642,31 @@ GenerateButtonEvent(
     return true;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpWarpPointer --
+ *
+ *	Move the mouse cursor to the screen location specified by the warpX and
+ *	warpY fields of a TkDisplay.
+ *
+ * Results:
+ *	None
+ *
+ * Side effects:
+ *	The mouse cursor is moved.
+ *
+ *----------------------------------------------------------------------
+ */
+
 void
 TkpWarpPointer(
     TkDisplay *dispPtr)
 {
     CGPoint pt;
-    UInt32 buttonState;
 
     if (dispPtr->warpWindow) {
 	int x, y;
-
 	Tk_GetRootCoords(dispPtr->warpWindow, &x, &y);
 	pt.x = x + dispPtr->warpX;
 	pt.y = y + dispPtr->warpY;
@@ -556,19 +675,55 @@ TkpWarpPointer(
 	pt.y = dispPtr->warpY;
     }
 
-    /*
-     * Tell the OSX core to generate the events to make it happen.
-     */
-
-    buttonState = [NSEvent pressedMouseButtons];
-    CGEventType type = kCGEventMouseMoved;
-    CGEventRef theEvent = CGEventCreateMouseEvent(NULL,
-						  type,
-						  pt,
-						  buttonState);
     CGWarpMouseCursorPosition(pt);
-    CGEventPost(kCGHIDEventTap, theEvent);
-    CFRelease(theEvent);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpSetCapture --
+ *
+ *	This function captures the mouse so that all future events will be
+ *	reported to this window, even if the mouse is outside the window. If
+ *	the specified window is NULL, then the mouse is released.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Sets the capture flag and captures the mouse.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpSetCapture(
+    TkWindow *winPtr)		/* Capture window, or NULL. */
+{
+    while (winPtr && !Tk_IsTopLevel(winPtr)) {
+	winPtr = winPtr->parentPtr;
+    }
+    captureWinPtr = (Tk_Window)winPtr;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkMacOSXGetCapture --
+ *
+ * Results:
+ *	Returns the current grab window
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Tk_Window
+TkMacOSXGetCapture(void)
+{
+    return captureWinPtr;
 }
 
 /*
