@@ -224,6 +224,7 @@
 #define DEF_LISTBOX_HIGHLIGHT_BG	NORMAL_BG
 #define DEF_LISTBOX_HIGHLIGHT		HIGHLIGHT
 #define DEF_LISTBOX_HIGHLIGHT_WIDTH	"1"
+#define DEF_LISTBOX_JUSTIFY		"left"
 #define DEF_LISTBOX_RELIEF		"sunken"
 #define DEF_LISTBOX_SCROLL_COMMAND	""
 #define DEF_LISTBOX_LIST_VARIABLE	""
@@ -246,7 +247,7 @@
 #define DEF_MENU_ENTRY_ACTIVE_FG	((char *) NULL)
 #define DEF_MENU_ENTRY_ACCELERATOR	((char *) NULL)
 #define DEF_MENU_ENTRY_BG		((char *) NULL)
-#define DEF_MENU_ENTRY_BITMAP		None
+#define DEF_MENU_ENTRY_BITMAP		NULL
 #define DEF_MENU_ENTRY_COLUMN_BREAK	"0"
 #define DEF_MENU_ENTRY_COMMAND		((char *) NULL)
 #define DEF_MENU_ENTRY_COMPOUND 	"none"
@@ -370,6 +371,7 @@
 #define DEF_PANEDWINDOW_HEIGHT		""
 #define DEF_PANEDWINDOW_OPAQUERESIZE	"1"
 #define DEF_PANEDWINDOW_ORIENT		"horizontal"
+#define DEF_PANEDWINDOW_PROXYBORDER	"2"
 #define DEF_PANEDWINDOW_RELIEF		"flat"
 #define DEF_PANEDWINDOW_SASHCURSOR	""
 #define DEF_PANEDWINDOW_SASHPAD		"0"
@@ -515,6 +517,14 @@
 
 #define DEF_CANVTEXT_FONT		"TkDefaultFont"
 
+/*
+ * Defaults for canvas items
+ * (arcs, bitmaps, lines, polygons, rectangles, and ovals):
+ */
+
+#define DEF_CANVBMAP_FG			NORMAL_FG
+#define DEF_CANVITEM_OUTLINE		NORMAL_FG
+
 /*
  * Defaults for toplevels (most of the defaults for frames also apply
  * to toplevels):
